var GLOBAL_BGM_VOLUME = 0.5;
var GLOBAL_SFX_VOLUME = 1;
var GLOBAL_LAST_LEVEL_ID = 24;

var gameLoopID;
var windowHeight = window.innerHeight;
var windowWidth = window.innerWidth;
var windowHalfWidth = window.innerWidth/2;
var caveContainerHeight;
var obtainedMineralsInCave = [];

var miner = document.getElementById("miner");
var minerDraw = document.getElementById("miner-draw");
var minerHeight = miner.getClientRects()[0].height;
var minerWidth = miner.getClientRects()[0].width;
var minerCoordY = miner.getClientRects()[0].y;
var minerCoordYH = miner.getClientRects()[0].y + miner.getClientRects()[0].height;
var joystick = document.getElementById("joystick");

var collisionDetected = false;
var nextFloorCounter = 0;
var floorsArray = [];
var floorsArrayLength = 0;
var floorCollisionMargin = minerWidth/2.5;
var specialCollisionableTiles;

var UISettingMusic = document.getElementById("ui-setting-music");
var UISettingSFX = document.getElementById("ui-setting-sfx");
var UISettingLanguage = document.getElementById("ui-setting-language");

var caveScreen = document.getElementById("cave-screen");
var levelSelectionScreen = document.getElementById("level-selection-screen");
var menuScreen = document.getElementById("menu-screen");
var storeScreen = document.getElementById("store-screen");
var endScreen = document.getElementById("end-screen");

var caveContainer = document.getElementById("cave-container");
var storeContainer = document.getElementById("store-container");
var levelsContainer = document.getElementById("levels-container");
var storeMineralsAmountElement = document.getElementById("store-minerals-amount");
var premiumModalStarsContainer = document.getElementById("premium-modal-stars-container");

var caveSuccessModal = document.getElementById("cave-success-modal");
var caveFailureModal = document.getElementById("cave-failure-modal");
var creditsModal = document.getElementById("credits-modal");
var settingsModal = document.getElementById("settings-modal");
var languageSelectorModal = document.getElementById("language-selector-modal");
var premiumModal = document.getElementById("premium-modal");
var tutorialModal = document.getElementById("tutorial-modal");

var darkMask = document.getElementById("dark-mask");

var BGMCave = document.getElementById("BGMCave");
BGMCave.volume = GLOBAL_BGM_VOLUME;
var SFXButton = document.getElementById("SFXButton");
SFXButton.volume = GLOBAL_SFX_VOLUME;
var SFXMineral = document.getElementById("SFXMineral");
SFXMineral.volume = GLOBAL_SFX_VOLUME;

joystick.addEventListener('touchstart', startMoveMiner);
joystick.addEventListener('touchmove', moveMiner, {passive: false});
joystick.addEventListener('touchend', releaseMiner);

levelsContainer.addEventListener("click", startNewGame, false);
storeContainer.addEventListener("click", buyProduct, false);
document.addEventListener("pause", gameAppPaused, false);
document.addEventListener("resume", gameAppResumed, false);
document.addEventListener("click", gameButtonPressed, false);

var currentScreen = menuScreen;
var currentModal = null;
var currentCaveId = null;
var currentCaveProperties = null;

function startNewGame(e){
    if(e.target.unlocked != 0){
        currentCaveProperties = e;
        createCave(e.target.caveId, caveArray[e.target.caveId], e.target.type, e.target.gravity);
    }
}

function buyProduct(e){
    var productPrice = e.target.price;
    var productId = e.target.productId;
    if(productPrice>=0){
        db.transaction(function(tx){
            tx.executeSql("SELECT minerals FROM playerTable", [], function(tx, res){
                var playerMinerals = res.rows.item(0).minerals;
                if(playerMinerals >= productPrice){
                    playerMinerals = playerMinerals - productPrice;
                    tx.executeSql("UPDATE storeTable SET obtained=? WHERE productId=?", [1, productId]);
                    tx.executeSql("UPDATE playerTable SET minerals=?", [playerMinerals]);
                    tx.executeSql("UPDATE playerTable SET skin=?", [productId]);
                }
            });
        }, function(error){alert(error);}, function(){
            loadStore(loadMinerals);
        });
    }
}

function openScreen(screen, callback, animation){
    if(callback){
        callback();
    }
    if(animation == "NONE"){
        currentScreen.style.visibility = "hidden";
        screen.style.visibility = "visible";
    }
    else if(animation == "MOVEMENT"){
        currentScreen.style.transform = "translateY(-100%)";
        screen.style.visibility = "visible";        
        screen.style.transform = "translateY(-100%)";      
    }
    currentScreen = screen;
}

function backToMenu(){
    currentScreen.style.transform = "translateY(0%)";
    menuScreen.style.transform = "translateY(0%)";
    currentScreen = menuScreen;
}

function backToMenuFromEnd(){
    levelSelectionScreen.style.transform = "translateY(0%)";
    levelSelectionScreen.style.transition = "none";
    currentScreen.style.visibility = "hidden";
    menuScreen.style.transform = "translateY(0%)";
    levelSelectionScreen.style.transition = "";
    currentScreen = menuScreen;    
}

function openModal(modal, callback){
    if(callback){
        callback();
    }
    currentModal = modal;
    darkMask.classList.add("ui-mask-active");
    modal.classList.add("ui-modal-active");
}

function closeModal(callback){
    if(callback){
        callback();
    }
    darkMask.classList.remove("ui-mask-active");
    currentModal.classList.remove("ui-modal-active");
}

function loadCaves(callback){
    db.transaction(function(tx){
        tx.executeSql("SELECT cT.ordered, cT.caveId, cT.type, cT.gravity, cT.unlocked, cT.completed, cMT.obtainedMinerals, cMT.maxMinerals FROM caveTable cT INNER JOIN (SELECT caveId, CASE WHEN obtainedMinerals IS NULL THEN 0 ELSE obtainedMinerals END obtainedMinerals, maxMinerals FROM ((SELECT caveId, COUNT(mineralId) maxMinerals FROM caveMineralsTable GROUP BY caveId ORDER BY caveId, mineralId ASC) A LEFT OUTER JOIN (SELECT caveId caveIDB, COUNT(mineralId) obtainedMinerals FROM caveMineralsTable WHERE obtained=1 GROUP BY caveId ORDER BY caveId, mineralId ASC) B ON A.caveId=B.caveIDB)) cMT ON cT.caveId=cMT.caveId GROUP BY cMT.caveId ORDER BY cT.ordered ASC", [], function(tx, res){
            levelsContainer.innerHTML = "";
            for(var i=0; i<res.rows.length; i++){
                var level = document.createElement("DIV");
                level.className = "level cave-type-" + res.rows.item(i).type;
                level.caveId = res.rows.item(i).caveId;
                level.type = res.rows.item(i).type;
                level.gravity = res.rows.item(i).gravity;
                level.unlocked = res.rows.item(i).unlocked;
                if(res.rows.item(i).unlocked == 0){
                    level.classList.toggle("level-locked");
                }
                var levelName = document.createElement("P");
                levelName.className = "level-name";
                levelName.innerText = GLOBAL_UI_TRANSLATION[lang]["UIS021"] + " " + res.rows.item(i).caveId; 
                level.appendChild(levelName);
                var mineralsCounter = document.createElement("DIV");
                mineralsCounter.className = "minerals-counter";
                var mineralsCounterDraw = document.createElement("IMG");
                mineralsCounterDraw.className = "minerals-counter-draw";
                if(res.rows.item(i).obtainedMinerals == res.rows.item(i).maxMinerals){
                    mineralsCounterDraw.src = "assets/icons/selected.png";
                }
                else{
                    mineralsCounterDraw.src = "assets/icons/minerals.png";
                }
                var mineralsCounterText = document.createElement("P");
                mineralsCounterText.textContent = res.rows.item(i).obtainedMinerals + " / " + res.rows.item(i).maxMinerals;
                mineralsCounter.appendChild(mineralsCounterDraw);
                mineralsCounter.appendChild(mineralsCounterText);
                level.appendChild(mineralsCounter);
                levelsContainer.appendChild(level);
            }
        });
    }, function(error){alert(error);}, function(){
        if(callback){
            callback();
        }
    });
}

function loadMinerals(callback){
    db.transaction(function(tx){
        tx.executeSql("SELECT minerals FROM playerTable", [], function(tx, res){
            storeMineralsAmountElement.innerText = res.rows.item(0).minerals;
        });
    }, function(error){alert(error);}, function(){
        if(callback){
            callback();
        }
    });
}

function loadStore(callback){
    var currentSkin;
    db.transaction(function(tx){
        tx.executeSql("SELECT skin FROM playerTable", [], function(tx, res){
           currentSkin = res.rows.item(0).skin;
        });
        tx.executeSql("SELECT * FROM storeTable ORDER BY productId ASC", [], function(tx, res){
            storeContainer.innerHTML = "";
            for(var i=0; i<res.rows.length; i++){
                var storeItemContainer = document.createElement("DIV");
                storeItemContainer.className = "store-item-container";
                var storeItemDrawContainer = document.createElement("DIV");
                storeItemDrawContainer.className = "store-item-draw-container";
                var storeItemPriceContainer = document.createElement("DIV");
                storeItemPriceContainer.className = "store-item-price-container ui-sound-button";
                storeItemPriceContainer.productId = res.rows.item(i).productId;
                var storeItemPriceDraw = document.createElement("IMG");
                storeItemPriceDraw.className = "store-item-price-draw";
                var storeItemDraw = document.createElement("IMG");
                storeItemDraw.className = "store-item-draw";
                storeItemDraw.src = "css/backgrounds/miners/ch-miner-" + res.rows.item(i).productId + ".png";
                var storeItemPriceValue = document.createElement("P");
                storeItemPriceValue.className = "store-item-price-value";
                if(res.rows.item(i).obtained==1){
                    if(res.rows.item(i).productId==currentSkin){
                        storeItemPriceValue.textContent = "";
                        storeItemPriceDraw.src = "assets/icons/selected.png";
                    }
                    else{
                        storeItemPriceDraw.style.display = "none";
                        storeItemPriceValue.textContent = GLOBAL_UI_TRANSLATION[lang]["UIS009"];
                        storeItemPriceValue.classList.add("ui-translate");
                    }
                    storeItemPriceContainer.price = 0;
                }
                else{
                    storeItemPriceDraw.src = "assets/icons/minerals.png";
                    storeItemPriceValue.textContent = res.rows.item(i).price;
                    storeItemPriceContainer.price = res.rows.item(i).price;
                }
                storeItemPriceContainer.appendChild(storeItemPriceDraw);
                storeItemPriceContainer.appendChild(storeItemPriceValue);
                storeItemDrawContainer.appendChild(storeItemDraw);
                storeItemContainer.appendChild(storeItemDrawContainer);
                storeItemContainer.appendChild(storeItemPriceContainer);
                storeContainer.appendChild(storeItemContainer);
            }
        });
    }, function(error){alert(error);}, function(){
        if(callback){
            callback();
        }
    });
}

function loadSettings(callback){
    db.transaction(function(tx){
        tx.executeSql("SELECT music, sfx, lang FROM settingsTable", [], function(tx, res){
            var musicPreference = res.rows.item(0).music;
            var sfxPreference = res.rows.item(0).sfx;
            var langPreference = res.rows.item(0).lang;
            if(musicPreference==0){
                UISettingMusic.innerText = GLOBAL_UI_TRANSLATION[lang]["UIS015"];
            }
            else{
                UISettingMusic.innerText = GLOBAL_UI_TRANSLATION[lang]["UIS014"];
            }
            if(sfxPreference==0){
                UISettingSFX.innerText = GLOBAL_UI_TRANSLATION[lang]["UIS015"];
            }
            else{
                UISettingSFX.innerText = GLOBAL_UI_TRANSLATION[lang]["UIS014"];
            }
            UISettingLanguage.innerText = GLOBAL_UI_SUPPORTED_LANGUAGES[lang];
            toggleGameMusic(musicPreference);
            toggleGameSFX(sfxPreference);
        });
        
    }, function(error){alert(error);}, function(){
        if(callback){
            callback();
        }
    });
}

function loadPremiumInfo(callback){
    db.transaction(function(tx){
        tx.executeSql("SELECT ads, premiumstars FROM settingsTable", [], function(tx, res){
            var ads = res.rows.item(0).ads;
            var premiumStars = res.rows.item(0).premiumstars;
            var starGoldSrc = "assets/icons/star-gold.png";
            var starGreySrc = "assets/icons/star-grey.png";
            premiumModalStarsContainer.innerHTML = "";
            for(var i=0; i<3; i++){
                var star = document.createElement("IMG");
                star.className = "premium-star";
                if(i<premiumStars){
                    star.src = starGoldSrc;
                }
                else{
                    star.src = starGreySrc;                    
                }
                premiumModalStarsContainer.appendChild(star);
            }
            if(ads==0){
                admob.banner.remove();
            }
            else{
                admob.banner.show();
            }
        });
    }, function(error){alert(error);}, function(){
        if(callback){
            callback();
        }
    });
}

function handlePremium(callback){
    db.transaction(function(tx){
        tx.executeSql("SELECT ads, premiumstars FROM settingsTable", [], function(tx, res){
            if(isRewardVideoAd == true){
                admob.rewardvideo.show();
                toggleGameMusic(0);
            }
        });
    }, function(error){alert(error);}, function(){
        if(callback){
            callback();
        }
    });
}

function startMoveMiner(e){
    joystick.style.visibility = "hidden";
}

function moveMiner(e){
    if(collisionDetected == false){
        var pageX = e.changedTouches[0].pageX;
        var movement = pageX - windowHalfWidth;
        if(pageX > windowHalfWidth){
            miner.style.transform = "translateX(" + movement + "px) rotateY(180deg)";
        }
        else{
            miner.style.transform = "translateX(" + movement + "px) rotateY(0deg)";
        }
        joystick.style.transform = "translateX(" + movement + "px)";
    }
    e.preventDefault();    
}

function releaseMiner(){  
    if(gameLoopID != null){
        joystick.style.visibility = "visible";        
    }
}

function cleanCave(){
    collisionDetected = false;
    nextFloorCounter = 0;
    obtainedMineralsInCave = [];
    caveContainer.style.transition = null;
    caveContainer.style.transition = "";
    caveContainer.offsetHeight;
    caveContainer.style.transform = "";
    caveContainer.innerHTML = "";
    caveContainer.offsetHeight;
    miner.style = "";
    joystick.style = "";
    miner.style.transform = "";
    miner.style.visibility = "hidden";
    joystick.style.visibility = "hidden";
}

function createCave(caveID, cave, typeOfCave, gravity){
    var caveToCreate = cave.slice();
    var caveSpeed = Math.round(cave.length/5);
    cleanCave();
    db.transaction(function(tx){
        tx.executeSql("SELECT * FROM caveMineralsTable WHERE caveID=? AND obtained=0", [caveID], function(tx, res){
            for(var i=0; i<res.rows.length; i++){
                caveToCreate[res.rows.item(i).positionY] = cave[res.rows.item(i).positionY].slice();
                caveToCreate[res.rows.item(i).positionY][res.rows.item(i).positionX] = 1;
            }
        });
        tx.executeSql("SELECT skin FROM playerTable", [], function(tx, res){
            minerDraw.style.background = "url('css/backgrounds/miners/ch-miner-" + res.rows.item(0).skin + ".png') no-repeat center";
            minerDraw.style.backgroundSize = "contain";
        });
    }, function(error){alert(error)}, function(){
        windowHeight = window.innerHeight;
        windowWidth = window.innerWidth;
        windowHalfWidth = window.innerWidth/2;
        minerCoordYH = miner.getClientRects()[0].y + miner.getClientRects()[0].height;
        caveScreen.style.backgroundImage = "url('css/textures/bg-" + typeOfCave + ".png')";
        caveContainer.style.transition = "transform " + caveSpeed + "s linear";
        for (var i=0; i<caveToCreate.length; i++){
            var floor = document.createElement("DIV");
            floor.className = "floor";
            for(var j=0; j<caveToCreate[i].length; j++){
                var floorTile = document.createElement("DIV");
                var typeOfTile = parseInt(caveToCreate[i][j].toString().substr(0,1)); // 0=Air, 1=Mineral, 2=Decoration, 3=Floor, 4=Background, 5=Especial
                floorTile.floorId = caveToCreate[i][j];
                floorTile.className = "floor-tile";
                floorTile.classList.add("floor-tile-" + caveToCreate[i][j]);
                if(typeOfTile==1 || typeOfTile==3){
                    floorTile.classList.add("floor-tile-collisionable");
                }
                else if(typeOfTile==5){
                    floorTile.classList.add("floor-tile-special-collisionable");
                }
                floorTile.typeOfTile = typeOfTile;
                floorTile.positionX = j;
                floorTile.positionY = i;
                floor.appendChild(floorTile);
            }
            caveContainer.appendChild(floor);
        }
        floorsArray = document.getElementsByClassName("floor");
        specialCollisionableTiles = document.getElementsByClassName("floor-tile-special-collisionable");
        floorsArrayLength = floorsArray.length;
        caveContainerHeight = caveContainer.getClientRects()[0].height;
        currentCaveId = caveID;
        openScreen(caveScreen, null, 'NONE');
        miner.style.visibility = "visible";
        joystick.style.visibility = "visible";
        caveContainer.style.transform = "translateY(-" + (caveContainerHeight + windowHeight) + "px)";
        gameLoopID = requestAnimationFrame(gameLoop);
    });
}

function gameLoop() {
    var minerCoordX = miner.getClientRects()[0].x;
    var minerCoordXW = minerCoordX + minerWidth;
    var nextFloor = floorsArray[nextFloorCounter];
    var nextFloorTiles = nextFloor.getElementsByClassName("floor-tile-collisionable");
    var allCollisionableTiles = [];
    allCollisionableTiles = Array.prototype.concat.apply(allCollisionableTiles, nextFloorTiles);
    allCollisionableTiles = Array.prototype.concat.apply(allCollisionableTiles, specialCollisionableTiles);
    var floorCoordY = nextFloor.getClientRects()[0].y;
    var floorCoordYH = floorCoordY + nextFloor.getClientRects()[0].height + floorCollisionMargin;
    if(minerCoordYH >= floorCoordY){
        for(var j=0; j<allCollisionableTiles.length; j++){
            var nextCollisionTileClientRects = allCollisionableTiles[j].getClientRects()[0];
            var nextCollisionTileX = nextCollisionTileClientRects.x;
            var nextCollisionTileY = nextCollisionTileClientRects.y;
            var nextCollisionTileXW = nextCollisionTileX + nextCollisionTileClientRects.width;
            var nextCollisionTileYH = nextCollisionTileY + nextCollisionTileClientRects.height;
            if((((minerCoordX >= nextCollisionTileX) && (minerCoordX <= (nextCollisionTileXW - floorCollisionMargin))) || ((minerCoordX <= nextCollisionTileX) && ((minerCoordXW - floorCollisionMargin) >= nextCollisionTileX))) && ((minerCoordYH >= nextCollisionTileY + floorCollisionMargin) && (minerCoordY <= (nextCollisionTileYH - floorCollisionMargin)))){
                if(allCollisionableTiles[j].typeOfTile == 1){
                    SFXMineral.pause();
                    SFXMineral.currentTime = 0;
                    SFXMineral.play();
                    var mineralObject = {
                        caveID: currentCaveId,
                        positionX: allCollisionableTiles[j].positionX,
                        positionY: allCollisionableTiles[j].positionY
                    }
                    allCollisionableTiles[j].style.visibility = "hidden";
                    obtainedMineralsInCave.push(mineralObject);
                    allCollisionableTiles[j].typeOfTile = 0;
                }
                if(allCollisionableTiles[j].typeOfTile == 3 || allCollisionableTiles[j].typeOfTile == 5){
                    collisionDetected = true;
                    break;
                }
            }
        }
    }
    if(collisionDetected == false){
        if(minerCoordYH > floorCoordYH){
            nextFloorCounter++;
        }
        if(nextFloorCounter == (floorsArrayLength-1)){
            caveCompleted();
        }
        else{
            gameLoopID = requestAnimationFrame(gameLoop);
        }
    }
    else{
        caveFailed();
    }
}

function caveCompleted(){
    db.transaction(function(tx){
        var obtainedMineralsAmount = obtainedMineralsInCave.length;
        for(var i=0; i<obtainedMineralsInCave.length; i++){
            tx.executeSql("UPDATE caveMineralsTable SET obtained=1 WHERE caveId=? AND positionX=? AND positionY=?", [obtainedMineralsInCave[i].caveID, obtainedMineralsInCave[i].positionX, obtainedMineralsInCave[i].positionY]);
        }
        tx.executeSql("UPDATE caveTable SET unlocked=1 WHERE ordered=(SELECT ordered+1 FROM caveTable WHERE caveId=?)", [currentCaveId], function(tx, res){});
        tx.executeSql("UPDATE playerTable SET caveLevel=?", [currentCaveId+1], function(tx, res){});
        tx.executeSql("UPDATE playerTable SET minerals=minerals+?", [obtainedMineralsAmount]);
    }, function(error){alert(error)}, function(){
        cancelAnimationFrame(gameLoopID);
        cleanCave();
        gameLoopID = null;
        if(currentCaveId==GLOBAL_LAST_LEVEL_ID){
            openScreen(endScreen, null, 'NONE');
        }
        else{
            openModal(caveSuccessModal);
        }
        loadCaves(loadMinerals);
    });
}

function caveFailed(){
    showDeath();
    cancelAnimationFrame(gameLoopID);
    gameLoopID = null;
    openModal(caveFailureModal);
}

function showDeath(){
    var caveContainerCurrentPosition = caveContainer.getClientRects()[0].y - 656;
    caveContainer.style.transform = "translateY(" + caveContainerCurrentPosition + "px)";
    minerDraw.style.background = "url('css/backgrounds/bg-death-marker.png') no-repeat center";
    minerDraw.style.backgroundSize = "contain";
    joystick.style.visibility = "hidden";
}

function retryCave(){
    startNewGame(currentCaveProperties);
    closeModal();
}

function changeSettings(element, setting){
    if(setting=="music"){
        db.transaction(function(tx){
            tx.executeSql("SELECT music FROM settingsTable", [], function(tx, res){
                var musicPreference = res.rows.item(0).music;
                if(musicPreference == 0){
                    musicPreference = 1;
                    toggleGameMusic(musicPreference);
                    element.innerText = GLOBAL_UI_TRANSLATION[lang]["UIS014"];
                }
                else{
                    musicPreference = 0;
                    toggleGameMusic(musicPreference);
                    element.innerText = GLOBAL_UI_TRANSLATION[lang]["UIS015"];
                }
                tx.executeSql("UPDATE settingsTable SET music=?", [musicPreference]);
            });
        }, 
        function(error){alert(error);}, function(){});
    }
    if(setting=="sfx"){
        db.transaction(function(tx){
            tx.executeSql("SELECT sfx FROM settingsTable", [], function(tx, res){
                var sfxPreference = res.rows.item(0).sfx;
                if(sfxPreference == 0){
                    sfxPreference = 1;
                    toggleGameSFX(sfxPreference);
                    element.innerText = GLOBAL_UI_TRANSLATION[lang]["UIS014"];
                }
                else{
                    sfxPreference = 0;
                    toggleGameSFX(sfxPreference);
                    element.innerText = GLOBAL_UI_TRANSLATION[lang]["UIS015"];
                }
                tx.executeSql("UPDATE settingsTable SET sfx=?", [sfxPreference]);
            });
        }, function(error){alert(error);}, function(){});
    }
    if(setting=="language"){
        closeModal();
        openModal(languageSelectorModal);
    }
}

function changeLanguage(language){
    db.transaction(function(tx){
        lang = language;
        tx.executeSql("UPDATE settingsTable SET lang=?", [lang], function(tx, res){});
    }, function(error){alert(error);}, function(){
        UISettingLanguage.innerText = GLOBAL_UI_SUPPORTED_LANGUAGES[lang];
        translateUI(lang);
        closeModal();
    });
}

function toggleGameMusic(decision){
    if(decision == 1){
        BGMCave.loop = true;
        BGMCave.play();
    }
    else{
        BGMCave.pause();
    }
}

function toggleGameSFX(decision){
    if(decision == 1){
        SFXButton.volume = GLOBAL_SFX_VOLUME;
        SFXMineral.volume = GLOBAL_SFX_VOLUME;
    }
    else{
        SFXButton.volume = 0;
        SFXMineral.volume = 0;
    }
}

function gameButtonPressed(e){
    if(e.target.classList.contains("ui-sound-button")){
        SFXButton.pause();
        SFXButton.currentTime = 0;
        SFXButton.play();
    }
}

function gameAppPaused(){
    toggleGameMusic(0);
    if(currentScreen==caveScreen){
        caveFailed();
    }
}

function gameAppResumed(){
    loadSettings();
}
